# The function proposed here - PhySignalPheno - tests phylogenetic signal in plant phenology considering the circularity of temporal data. Any query or suggestion please write to v.staggemeier@gmail.com. 

# For details please see the file	PhylogSignal_CircData.pdf - it contains a short theoretical background and explanations about the packages that must be loaded, the input datasets, the arguments required by the function, and all results created with the function PhySignalPheno. Some suggestions to elaborate graphics are also available in the PhylogSignal_CircData.pdf.

# Please note that all analyses were conducted in R version 3.3.2, RStudio Version 1.1.447, and PVR package Version: 0.2.1 (this package is provided in Dryad because the present version in CRAN has problems).

# This function requires four arguments, they are:
##     1. phy: time-calibrated molecular phylogenetic hypothesis;
##     2. dates: the file with phenological dates (first dates or peak dates);
##     3. year: the year of study;
##     4. name: the name of the community, the results saved in a separate file will use this name. Use letters without spaces or symbols.


#   This function produce a list containing:
##     1. Phylogenetic signal for all four metrics;
##     2. PCoA results;
##     3. P-value for selected vectors in PVR analysis;
##     4. Mantel results;
##     5. Angles (degrees) and Dates (DOY);
# The function also produces a file that will be saved in your working directory, remember to set the working directory before starting the analyses. 

#After install the required packages (listed below), load all of them:

library("phytools")
library("picante")
library("phylobase")
library("adephylo")
library("circular")
library("PVR")
library("lubridate")
library("ecodist")

setwd("write here the path to your working directory") # setting the working directory folder

#########################
# Loading the function  #
#########################
 
 PhySignalPheno <- function(phy, dates, year=NULL, name=NULL){
      #verify whether the phylogenetic hypothesis is ok
      if(!is.binary.tree(phy)){ stop("phylogenies must be binary")}
      if(!is.ultrametric(phy)){stop("phylogenies must be ultrametric")}
      # to convert dates in angles
      leap_year(year) # true or false to identify leap years
      # for a leap year, the angles will be 360/366=0.9836065573770492, if not
      # 360/365=0.9863013698630137
      
      # Matching Phylo and Pheno(dates) # IMPORTANT STEP to compare taxa present in phylogeny with dates of each species, pruning and sorting the two kinds of data to match one another for subsequent analysis 
      # Species names must be written identically in both the phylogeny and the phenological datasets.
      phylo_traits<-match.phylo.data(phy, dates)
      phy <- phylo_traits$phy
      dates<-phylo_traits$data
      angles<-matrix(NA, nrow(dates), ncol(dates))
      for (i in 1:nrow(dates)) {
        angles[i, 1] <- round(
          ifelse(leap_year(year) == "TRUE",
                 dates[i, 1] * 0.9863013698630137,
                 dates[i, 1] * 0.9836065573770492
          ),3)
      }
      rownames(angles)<-row.names(dates)
      matriz.Y <- matrix(angles, nrow = length(angles),
                         ncol = length(angles),
                         byrow = FALSE)
      rownames(matriz.Y)<-rownames(angles)
      colnames(matriz.Y)<-rownames(angles)
      step_1<-matriz.Y-t(matriz.Y) #difference between each pair of species
      step_2<-abs(step_1) # transforming differences to absolute values
      step_3<-ifelse(step_2>180, step_2-360, step_2)
      step_4<-abs(step_3) # transforming to absolute values
      dist_angular<-as.dist(step_4) # matrix showing the angular distances 
      # between species
      pcoa<-dudi.pco(dist_angular, scannf = FALSE, nf = 2)  # Principal Coordinate 
      # Analysis to extract the scores that represent the angles on a linear scale
      explic.ax1<-pcoa$eig[1]/sum(pcoa$eig)
      explic.ax2<-pcoa$eig[2]/sum(pcoa$eig)
      scores_PCoA<-round(pcoa$li, 3)
      rownames(scores_PCoA)<-rownames(angles)
      # Matching Phylo and Pheno(scores)
      phylo_traits<-match.phylo.data(phy, scores_PCoA)
      phy <- phylo_traits$phy
      scores <- phylo_traits$data
      colnames(scores)<-colnames(scores_PCoA)
      
      
      ### Phylogenetic signal calculations ###
      
      
      pvr.scores <- PVRdecomp(phy, scale=TRUE)
      PVR.r.scores <- PVR(pvr.scores,phy, scores[,1],
                          method = "moran", scaled=TRUE)
      PVR.r2.scores <- round(PVR.r.scores@PVR$R2, digits=3)
      PVR_model.scores<-lm(scores[,1] ~ PVR.r.scores@Selection$Vectors)
      PVR.r.scores@Selection$Id
      p.value.PVR.scores<-
        round(pf(summary(PVR_model.scores)$fstatistic[1],
                 summary(PVR_model.scores)$fstatistic[2],
                 summary(PVR_model.scores)$fstatistic[3],
                 lower.tail = F),4)
      RESU <- matrix(NA, nrow=nrow(scores), 3+(ncol(PVR.r.scores@Selection$Id)))
      RESU <- cbind(dates, scores, angles, PVR.r.scores@Selection$Vectors)  
      write.table(RESU, paste("pheno_phylo_variables_", name, ".txt", sep = ""))
      # This table contains: DOY, Phenological Scores, Angles, Phylogenetic vectors selected in PVR analysis.
      
      
      ### Mantel ###
      
      dist.p <- as.dist(cophenetic(phy)) # phylogenetic distances
      dist.y <- dist_angular # phenological distances
      mantel.r.angle <-
        round(vegan::mantel(dist.y, dist.p, permutation = 1000)$statistic,
              digits = 3)
      mantel.r.signif.angle <-
        round(vegan::mantel(dist.y, 
                            dist.p, 
                            permutation = 1000)$signif,
              digits = 3)
      set.seed(5)
      correl_mantel_flor<-mgram(dist.p,dist.y, nperm = 999)
      
      
      ### Blomberg`s K ###
      
      scores_1 <- scores[, 1]     
      names(scores_1) <- phy$tip.label      
      K_circularax1<-phylosig(phy, scores_1, method="K", test=TRUE, nsim=1000)
      K_circularax1.K <-round(K_circularax1$K, digits=3)
      K_circularax1.p<-round(K_circularax1$P, digits=3)
      
      
      ### Pagel's Lambda ###
      
      lambda <- phylosig(phy, scores_1, method="lambda", test=TRUE, nsim=1000)
      lambda.val <- round(lambda$lambda, digits = 3)
      lambda.p <- round(lambda$P, digits = 3)
      
      return(list(
        Phylogenetic_signal = 
          c(
            pvr.scores=PVR.r2.scores, 
            p.PVR.scores=p.value.PVR.scores, 
            mantel.Correl=mantel.r.angle, 
            mantel.sig=mantel.r.signif.angle, 
            K=K_circularax1.K , K.p=K_circularax1.p, 
            lambda = lambda.val, lambda.p = lambda.p), 
        Angular_PCOA_axis = pcoa,
        selected.vectors.PVR = PVR.r.scores@Selection$Id,
        Mantel = list(Distances = cbind(dist.y = dist.y, 
                                        dist.p = dist.p), 
                      Correlogram = correl_mantel_flor),
        Angles_and_Dates = list(angles = angles, dates = dates)))
    }
 
 #########################
 # Loading your data
 #########################
 # The input files used here are available in Dryad
 # When using your data remember to set your working directory

 # # Phylogenetic hypothesis
 phy<-read.nexus("tree_example.nex") # Change here to select your input file - phylogeny

 # # First flowering dates
 FFD<-read.table("pheno_example1.txt") # Change here to select your input file - phenological dataset
 
 # Please note that regular years (i.e. 365 days) and leap years (366 days) 
 # result in different angular transformations when converting from DOY 
 # (day of the year) to angles, so you must input the year of your study.
 # In case you do not have the mean date per species, but you do have the mean date per 
 # individual, you can use the lubridate R package to convert the individual dates 
 # into angles and estimate the mean date per species using the circular package 
 # before to run the analyses of PhySignalPheno function.
 
 year<-2018 # change here for the year of your study
 dates<-FFD
 
 #########################
 # Running the function  #
 #########################
 # Please, use the PVR version available in Dryad
 resu_example1 <-
   PhySignalPheno(phy, FFD, year = 2018, name = c("example_1"))
 resu_example1 
 resu_example1$Phylogenetic_signal
 
 
 #############################
 # Some graphics as examples #
 #############################
  
    dates<-resu_example1$Angles_and_Dates$dates
  pheno_angles <- circular(
    resu_example1$Angles_and_Dates$angles,
    type = "angles",
    units = "degrees",
    rotation = "clock",
    modulo = "2pi"
  )
  
  on <- par(mfrow = c(1, 2))
  hist(
    resu_example1$Angles_and_Dates$dates[, 1],
    main = "Flowering dates (DOY)",
    cex.main = 0.8,
    xlim = c((min(dates) - 30), max(dates) + 30),
    border = "plum4",
    col = "plum",
    xlab = "DOY: day of the year",
    cex = 1
  )
  
  
  plot(
    pheno_angles,
    stack = TRUE,
    rotation = "clock",
    zero = pi / 2,
    cex = 1,
    axes = F,
    ticks = F,
    shrink = 1.2,
    col = "black"
  )
  
  rose.diag(
    pheno_angles,
    unit = "degrees",
    bins = 12,
    axes = TRUE,
    ticks = TRUE,
    tcl = 0.05,
    tcl.text = -0.16,
    border = "plum4",
    col = "plum",
    tol = 0.2,
    radii.scale = "linear",
    rotation = "clock",
    zero = pi / 2,
    prop = 2.0,
    lty = 1,
    lwd = 2.5,
    cex = 0.8,
    add = TRUE
  )
  
  axis.circular(
    at = circular(seq(pi / 12, 2 * pi, pi / 6)),
    labels = c(
      "Mar",
      "Feb",
      "Jan",
      "Dec",
      "Nov",
      "Oct",
      "Sep",
      "Aug",
      "Jul",
      "Jun",
      "May",
      "Apr"
    ),
    cex = 0.8,
    tcl.text = 0.20
  )
  title("Flowering dates (angles)", cex.main = 0.8)
  par(on)

  
    corDegrade_linear <- colorRampPalette(
      c(
        "midnightblue",
        "navyblue",
        "dodgerblue3",
        "lightskyblue",
        "chartreuse4",
        "yellowgreen",
        "yellow",
        "orange",
        "tomato1",
        "red3",
        "red4",
        "maroon1"
      ))
  
  plot1 <- phylo4d(phy, 
                   cbind(dates = resu_example1$Angles_and_Dates$dates, 
                         axis = resu_example1$Angular_PCOA_axis$li))
  
  table.phylo4d(
    plot1,
    ratio.tree = 0.968,
    var.label = c("Dates", 
                  "Scores of the first PCoA axis"),
    cex.symbol = 1,
    grid = FALSE,
    pch = 15,
    box = F,
    center = F,
    scale = F,
    symbol = "colors",
    col = corDegrade_linear(365),
    cex.label = 0.5,
    cex = 1,
    legend = TRUE,
    show.tip.label = FALSE
  )
  title("Phenological variables mapped over the tree", cex.main = 0.8)

    

    on <- par(
      mfrow = c(1, 2),
      oma = c(2, 2, 2, 1),
      mar = c(4, 4, 2, 1)
    )
  
  plot(
    resu_example1$Mantel$Distances[, 1] ~ 
      resu_example1$Mantel$Distances[, 2],
    xlab = "Phylogenetic distances",
    ylab = "Phenological distances",
    bty = "l",
    pch = 19,
    cex = 0.5
  )
  title(main = "Relationship between distances", 
        cex.main = 0.8)
  
  plot.mgram(
    resu_example1$Mantel$Correlogram,
    xlab = "Midpoint of the distance class",
    bty = "l",
    ylab = "Mantel r"
  )
  title(main = "Mantel correlogram", cex.main = 0.8)
  
  par(on)

    
   